#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name: __main__.py
# Author: Podalirius (@podalirius_)
# Date created: 24 May 2024

#!/usr/bin/env python3

import sys

# Local library imports
from smbclientng import console

if __name__ == "__main__":
    sys.exit(console.run())